@@ -1,6 +1,6 @@
 /*
  *	wxcomponents.h - Custom wxWidgets components header
- *	Copyright (C) 2011, D Haley 
+ *	Copyright (C) 2013, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -18,15 +18,14 @@
 
 #include "wxcomponents.h"
 #include "wxcommon.h"
-#include "basics.h"
+#include "common/stringFuncs.h"
+
+#include "common/constants.h"
+#include "common/translation.h"
 
-#include "translation.h"
 
 #include <wx/clipbrd.h>
-#include <wx/filefn.h>
 
-#include <string>
-#include <fstream>
 
 using std::ofstream;
 using std::vector;
@@ -37,7 +36,98 @@ const float FONT_HEADING_SCALEFACTOR=1.0f;
 const float FONT_HEADING_SCALEFACTOR=0.75f;
 #endif
 
+void upWxTreeCtrl(const FilterTree &filterTree, wxTreeCtrl *t,
+		std::map<size_t,Filter *> &filterMap,vector<const Filter *> &persistentFilters,
+		const Filter *visibleFilt)
+{
+	//Remove any filters that don't exist any more
+	for(unsigned int ui=persistentFilters.size();ui--;)
+	{
+		if(!filterTree.contains(persistentFilters[ui]))
+		{
+			std::swap(persistentFilters[ui],persistentFilters.back());
+			persistentFilters.pop_back();
+		}
+	}
 
+	stack<wxTreeItemId> treeIDs;
+	//Warning: this generates an event, 
+	//most of the time (some windows versions do not according to documentation)
+	t->DeleteAllItems();
+
+	//Clear the mapping
+	filterMap.clear();
+	size_t nextID=0;
+	
+	size_t lastDepth=0;
+	//Add dummy root node. This will be invisible to wxTR_HIDE_ROOT controls
+	wxTreeItemId tid;
+	tid=t->AddRoot(wxT("TreeBase"));
+	t->SetItemData(tid,new wxTreeUint(nextID));
+
+	// Push on stack to prevent underflow, but don't keep a copy, 
+	// as we will never insert or delete this from the UI	
+	treeIDs.push(tid);
+
+	nextID++;
+	std::map<const Filter*,wxTreeItemId> reverseFilterMap;
+	//Depth first  add
+	for(tree<Filter * >::pre_order_iterator filtIt=filterTree.depthBegin();
+					filtIt!=filterTree.depthEnd(); ++filtIt)
+	{	
+		//Push or pop the stack to make it match the iterator position
+		if( lastDepth > filterTree.depth(filtIt))
+		{
+			while(filterTree.depth(filtIt) +1 < treeIDs.size())
+				treeIDs.pop();
+		}
+		else if( lastDepth < filterTree.depth(filtIt))
+		{
+			treeIDs.push(tid);
+		}
+		
+
+		lastDepth=filterTree.depth(filtIt);
+	
+		//This will use the user label or the type string.	
+		tid=t->AppendItem(treeIDs.top(),
+			wxStr((*filtIt)->getUserString()));
+		t->SetItemData(tid,new wxTreeUint(nextID));
+	
+
+		//Record mapping to filter for later reference
+		filterMap[nextID]=*filtIt;
+		//Remember the reverse mapping for later in
+		// this function when we reset visibility
+		reverseFilterMap[*filtIt] = tid;
+
+		nextID++;
+	}
+
+	//Try to restore the  selection in a user friendly manner
+	// - Try restoring all requested filter's visibility
+	// - then restore either the first requested filter as the selection
+	//   or the specified parameter filter as the selection.
+	if(persistentFilters.size())
+	{
+		for(unsigned int ui=0;ui<persistentFilters.size();ui++)
+			t->EnsureVisible(reverseFilterMap[persistentFilters[ui]]);
+
+		if(!visibleFilt)
+			t->SelectItem(reverseFilterMap[persistentFilters[0]]);
+		else
+			t->SelectItem(reverseFilterMap[visibleFilt]);
+
+		persistentFilters.clear();
+	}
+	else if(visibleFilt)
+	{
+		ASSERT(reverseFilterMap.find(visibleFilt)!=reverseFilterMap.end())
+		t->SelectItem(reverseFilterMap[visibleFilt]);
+	}
+	t->GetParent()->Layout();
+
+}
 
 //Convert my internal choice string format to wx's
 std::string wxChoiceParamString(std::string choiceString)
@@ -76,12 +166,28 @@ std::string wxChoiceParamString(std::string choiceString)
 BEGIN_EVENT_TABLE(wxPropertyGrid, wxGrid)
 	EVT_GRID_CELL_LEFT_CLICK(wxPropertyGrid::OnCellLeftClick )
 	EVT_GRID_LABEL_LEFT_DCLICK(wxPropertyGrid::OnLabelDClick) 
+	EVT_MOTION(wxPropertyGrid::OnMouseMove) 
 END_EVENT_TABLE()
 
 wxPropertyGrid::wxPropertyGrid(wxWindow* parent, wxWindowID id, 
 		const wxPoint& pos , const wxSize& size , long style ) :
 					wxGrid(parent, id, pos, size , style)
 {
+	lastGridHoverCol=lastGridHoverRow=-1;
+	//Perform wx event connecting/binding
+#if wxCHECK_VERSION(2, 9, 0)
+	GetGridWindow()->Bind(wxEVT_MOTION, &wxPropertyGrid::OnMouseMove, this);
+#else
+    	GetGridWindow()->Connect(wxID_ANY,
+                 wxEVT_MOTION, wxMouseEventHandler(wxPropertyGrid::OnMouseMove), NULL, this);
+#endif
+}
+
+
+void wxPropertyGrid::setGroupName(unsigned int set, const std::string &name) 
+{
+	ASSERT(set < sectionNames.size());
+	sectionNames[set]=name;
 }
 
 void wxPropertyGrid::OnSize(wxSizeEvent &event)
@@ -91,6 +197,8 @@ void wxPropertyGrid::OnSize(wxSizeEvent &event)
 
 void wxPropertyGrid::OnLabelDClick(wxGridEvent &event)
 {
+	if(!this->GetCols())
+		return;
 	wxSize s;
 	s=this->GetSize();
 	fitCols(s);
@@ -107,6 +215,11 @@ void wxPropertyGrid::fitCols(wxSize &size)
 
 wxPropertyGrid::~wxPropertyGrid()
 {
+#if wxCHECK_VERSION(2, 9, 0)
+   GetGridWindow()->Unbind(wxEVT_MOTION, &wxPropertyGrid::OnMouseMove, this);
+#else
+   GetGridWindow()->Disconnect();
+#endif
 }
 
 //Function adapted from nomadsync.sourceforge.net (GPL)
@@ -181,6 +294,52 @@ void wxGridCellChoiceRenderer::Draw(wxGrid& grid, wxGridCellAttr& attr, wxDC& dc
 	}
 }
 
+bool wxPropertyGrid::isSeparatorRow(int row) const
+{
+	//TODO: Consider moving to map, this does not scale well.
+	// for now we have usually five or six items tho
+	return std::find(sepRows.begin(),sepRows.end(),row)!=sepRows.end();
+}
+
+void wxPropertyGrid::OnMouseMove(wxMouseEvent &event)
+{
+
+	int xPos,yPos;
+	int xMouse,yMouse;
+	event.GetPosition(&xMouse,&yMouse);
+
+	CalcUnscrolledPosition(xMouse,yMouse,&xPos,&yPos);
+
+	int row,col;
+	row=YToRow(yPos);
+	col=XToCol(xPos);
+
+	//Check if the column or row was different
+	if(col != lastGridHoverCol || row != lastGridHoverRow)
+	{
+		lastGridHoverRow = row;
+		lastGridHoverCol = col;
+
+		//If we are hovering over a "spacer" row,
+		//the null row, or not on the prop
+		//column, then bail out
+		if(isSeparatorRow(row) || row == -1  || col == 1)
+		{
+			SetToolTip(NULL); //Hide the tooltip
+			event.Skip();
+			return;
+		}
+
+		//Set the help text based upon the group property
+		const GRID_PROPERTY *p;
+		p=getProperty(getKeyFromRow(row));
+	
+		SetToolTip(wxStr(p->helpText));
+	}
+
+	event.Skip();
+}
+
 //Function adapted from nomadsync.sourceforge.net (GPL)
 void wxPropertyGrid::OnCellLeftClick(wxGridEvent &ev)
 {
@@ -208,6 +367,9 @@ void wxPropertyGrid::clear()
 	
 	propertyKeys.clear();
 	this->EndBatch();
+
+	SetToolTip(wxT(""));
+	lastGridHoverRow=lastGridHoverCol=-1;
 }
 
 bool wxPropertyGrid::isComboEditing() 
@@ -237,8 +399,9 @@ void  wxPropertyGrid::clearKeys()
 	sectionNames.clear();
 }
 
-void wxPropertyGrid::addKey(const std::string &name, unsigned int set,
-		unsigned int newKey, unsigned int type, const std::string &data)
+void wxPropertyGrid::addKey(const std::string &name, unsigned int group,
+		unsigned int newKey, unsigned int type, const std::string &data,
+		const std::string &helpText)
 {
 
 	GRID_PROPERTY newProp;
@@ -246,9 +409,8 @@ void wxPropertyGrid::addKey(const std::string &name, unsigned int set,
 	newProp.type=type;
 	newProp.name=name;
 	newProp.data=data;
-
-	propertyKeys[set].push_back(newProp);
-
+	newProp.helpText = helpText;
+	propertyKeys[group].push_back(newProp);
 }
 
 //Layout the property vector
@@ -266,7 +428,7 @@ void wxPropertyGrid::propertyLayout()
 	this->SetColLabelValue(0,wxTRANS("Param"));
 	this->SetColLabelValue(1,wxTRANS("Value"));
 
-	vector<int> sepRows;
+	sepRows.clear();
 	sepRows.resize(propertyKeys.size());
 	int rows=0;
 	for (unsigned int ui=0; ui<propertyKeys.size(); ui++)
@@ -282,7 +444,7 @@ void wxPropertyGrid::propertyLayout()
 		rows++;
 	}
 
-//Remove last separator row.
+	//Remove last separator row.
 	if (rows)
 		rows--;
 	this->AppendRows(rows);
@@ -327,7 +489,7 @@ void wxPropertyGrid::propertyLayout()
 			case PROPERTY_TYPE_COLOUR:
 				//OK, this is totally inefficient, and hacky. but set the colour
 				//based upon the colour string. Then when user edits, use a colour
-				//dialog (handled elswehere)
+				//dialog (handled elsewhere)
 				unsigned char r,g,b,a;
 				parseColString(propertyKeys[ui][uj].data,r,g,b,a);
 				attr->SetBackgroundColour(wxColour(r,g,b,a));
@@ -353,6 +515,12 @@ void wxPropertyGrid::propertyLayout()
 				wxGridCellChoiceEditor *choiceEd=new wxFastComboEditor(a,false);
 				this->SetCellEditor(propertyKeys[ui][uj].renderPosition,
 				                    1,choiceEd);
+
+				//Required to prevent wx from asserting,
+				// "GetEventHandler == this" failed in ~wxWindowBase(), any pushed event handlers must be removed".
+				// I posted on the wxwidgets forum and got the reply that I should try this, but they were not sure.
+				// seems to work
+				attr->DecRef();
 				break;
 			}
 			case PROPERTY_TYPE_STRING:
@@ -376,7 +544,7 @@ void wxPropertyGrid::propertyLayout()
 				this->SetCellValue(sepRows[ui],0,wxStr(sectionNames[ui+1]));
 				wxFont f;
 				f.SetStyle(wxFONTSTYLE_ITALIC);
-				f.SetPointSize(f.GetPointSize()*FONT_HEADING_SCALEFACTOR);
+				f.SetPointSize((int)(f.GetPointSize()*FONT_HEADING_SCALEFACTOR));
 				this->SetCellFont(sepRows[ui],0,f);
 			}
 
@@ -388,22 +556,23 @@ void wxPropertyGrid::propertyLayout()
 		}
 	}
 
+	//First column is read only
 	wxGridCellAttr *readOnlyGridAttr=new wxGridCellAttr;
 	readOnlyGridAttr->SetReadOnly(true);
 	SetColAttr(0,readOnlyGridAttr);
+	//Remove labels for column lead-in
 	SetRowLabelSize(0);
 	SetMargins(0,0);
 
 	AutoSizeColumn(0,true);
 	EndBatch();
 
-	//Expand the column contents	
+	//Expand the column contents to fit current size	
 	wxSize s;
 	s=GetSize();
 	fitCols(s);
 }
 
-
 unsigned int  wxPropertyGrid::getKeyFromRow(int row) const
 {
 	for(unsigned int ui=0;ui<propertyKeys.size();ui++)
@@ -420,31 +589,16 @@ unsigned int  wxPropertyGrid::getKeyFromRow(int row) const
 	return 0; // Shut gcc up
 }
 
-unsigned int wxPropertyGrid::getSetFromRow(int row) const
+const GRID_PROPERTY *wxPropertyGrid::getProperty(unsigned int key) const
 {
-	for(unsigned int ui=0;ui<propertyKeys.size();ui++)
+	for(unsigned int group=0;group<propertyKeys.size();group++)
 	{
-
-		for(unsigned int uj=0;uj<propertyKeys[ui].size();uj++)
+		for(unsigned int ui=0;ui<propertyKeys[group].size(); ui++)
 		{
-			if(propertyKeys[ui][uj].renderPosition == row)
-				return ui;
+			if(propertyKeys[group][ui].key == key)
+				return &(propertyKeys[group][ui]);
 		}
 	}
-
-	ASSERT(false);
-	return 0; // Shut gcc up
-}
-
-const GRID_PROPERTY *wxPropertyGrid::getProperty(unsigned int set,unsigned int key) const
-{
-	ASSERT(set < propertyKeys.size());
-	for(unsigned int ui=0;ui<propertyKeys[set].size(); ui++)
-	{
-		if(propertyKeys[set][ui].key == key)
-			return &(propertyKeys[set][ui]);
-	}
-
 	return 0;
 }
 
@@ -466,6 +620,9 @@ void wxFastComboEditor::BeginEdit(int row, int col, wxGrid* grid)
 #endif
 }
 
+BEGIN_EVENT_TABLE(CopyGrid, wxGrid)
+	EVT_KEY_DOWN(CopyGrid::OnKey) 
+END_EVENT_TABLE()
 CopyGrid::CopyGrid(wxWindow* parent, wxWindowID id, 
 		const wxPoint& pos , 
 		const wxSize& size , 
@@ -538,15 +695,8 @@ void CopyGrid::OnKey(wxKeyEvent &event)
 
 
 
-#ifndef __APPLE__
-        // If Ctrl+C is pressed...
-        if (event.ControlDown() && event.GetKeyCode() == 67)
+        if (event.CmdDown() && event.GetKeyCode() == 67)
 	{
-#else
-	//Apple is special and uses the meta key...
-        if (event.MetaDown() && event.GetKeyCode() == 67)
-	{
-#endif
 	
             copyData();
 	    return;
@@ -569,57 +719,155 @@ void CopyGrid::copyData()
 	wxGridCellCoordsArray arrayBR(GetSelectionBlockBottomRight());
 	
 
-	if(!arrayTL.Count() || !arrayBR.Count())
-		return;
-
-	wxGridCellCoords coordTL = arrayTL.Item(0);
-	wxGridCellCoords coordBR = arrayBR.Item(0);
-
-
-	rows = coordBR.GetRow() - coordTL.GetRow() +1;
-        cols = coordBR.GetCol() - coordTL.GetCol() +1;	
-
-        // data variable contain text that must be set in the clipboard
+	// data variable contain text that must be set in the clipboard
 	std::string data;
-        // For each cell in selected range append the cell value in the data
-	//variable
-    	// Tabs '\\t' for cols and '\\n' for rows
-    	for(int  r=0; r<rows; r++)
-	{
-            for(int  c=0; c<cols; c++)
-	    {
-                data+= stlStr(GetCellValue(coordTL.GetRow() + r,
-						coordTL.GetCol()+ c));
-                if(c < cols - 1)
-                    data+= "\t";
+	std::string endline;
+#ifdef __WXMSW__
+	endline="\r\n";
+#else
+	endline="\n";
+#endif
 
-	    }
-		#ifdef __WXMSW__
-		data+="\r\n";
-		#else
-	    data+="\n"; 
-		#endif
+	if(arrayTL.Count() && arrayBR.Count())
+	{
+
+		wxGridCellCoords coordTL = arrayTL.Item(0);
+		wxGridCellCoords coordBR = arrayBR.Item(0);
+
+
+		rows = coordBR.GetRow() - coordTL.GetRow() +1;
+		cols = coordBR.GetCol() - coordTL.GetCol() +1;	
+
+		//copy title from header
+		if(cols)
+		{
+			for(int ui=0;ui<cols;ui++)
+			{
+				data+=stlStr(GetColLabelValue(ui));
+
+				if(ui<cols-1)
+					data+="\t";
+			}
+			data+=endline;
+		}
+
+		// For each cell in selected range append the cell value in the data
+		//variable
+		// Tabs '\\t' for cols and '\\n' for rows
+		for(int  r=0; r<rows; r++)
+		{
+		    for(int  c=0; c<cols; c++)
+		    {
+			data+= stlStr(GetCellValue(coordTL.GetRow() + r,
+							coordTL.GetCol()+ c));
+			if(c < cols - 1)
+			    data+= "\t";
+
+		    }
+		    data+=endline;
+		}
 	}
-	// Create text data object
-	wxClipboard *clipboard = new wxClipboard();
+	else
+	{
+		const wxArrayInt& rows(GetSelectedRows());
+		const wxArrayInt& cols(GetSelectedCols());
+		const wxGridCellCoordsArray& cells(GetSelectedCells());
+
+		//Copy title from header
+		if(cols.size())
+		{
+			for(int ui=0;ui<cols.size();ui++)
+			{
+				data+=stlStr(GetColLabelValue(ui));
+
+				if(ui<cols.size()-1)
+					data+="\t";
+			}
+			data+=endline;
+
+			for(int ui=0;ui<GetNumberRows(); ui++)
+			{
+			    for(int  c=0; c<cols.size(); c++)
+			    {
+				data+= stlStr(GetCellValue(ui,cols[c]));
+				if(c < cols.size()-1)
+				    data+= "\t";
+
+			    }
+			    data +=endline;
+			}
+
+
+		}
+		else if (rows.size())
+		{
+			for(int ui=0;ui<GetNumberCols();ui++)
+			{
+				data+=stlStr(GetColLabelValue(ui));
+
+				if(ui<cols.size()-1)
+					data+="\t";
+			}
+			data+=endline;
+
+			for(int r=0;r<rows.size(); r++)
+			{
+			    for(int  c=0; c<GetNumberCols(); c++)
+			    {
+				data+= stlStr(GetCellValue(rows[r],c));
+				if(c < GetNumberCols()-1)
+				    data+= "\t";
+
+			    }
+			    data +=endline;
+			}
+		}
+/*		else if(cells.size())
+		{
+			//FIXME: Needs more thought than I have time for right now. 
+			// the problem is that cells[] doesn't neccesarily sort tl->br,
+			// so you have to do this first
+			int lastRow=cells[0].GetRow();
+			int lastCol=cells[0].GetCol();
+			for(int cell=0; cell<cells.size();cell++)
+			{
+
+				if(cells[cell].GetRow() > lastRow)
+				{
+					lastRow=cells[cell].GetRow();
+					data+=endline;
+				}
+				data+=stlStr(GetCellValue(cells[cell].GetRow(),
+							cells[cell].GetCol()));
+
+				if(lastCol < cells[cell].GetCol())
+				{
+					lastCol=cells[cell].GetCol();
+					data+="\t";
+				}
+			}
+		} */
+		else 
+			return;
+
+
+	}
+
 	// Put the data in the clipboard
-	if (clipboard->Open())
+	if (wxTheClipboard->Open())
 	{
 		wxTextDataObject* clipData= new wxTextDataObject;
 		// Set data object value
 		clipData->SetText(wxStr(data));
-		clipboard->UsePrimarySelection(false);
-		clipboard->SetData(clipData);
-		clipboard->Flush();
-		//This causes double free bug?
-		clipboard->Close();
+		wxTheClipboard->UsePrimarySelection(false);
+		wxTheClipboard->SetData(clipData);
+		wxTheClipboard->Close();
 	}
 
-	delete clipboard;
 }
 
 
-std::string TTFFinder::findFont(const char *fontFile) const
+std::string TTFFinder::findFont(const char *fontFile)
 {
 	//Action is OS dependant
 	
@@ -635,19 +883,59 @@ std::string TTFFinder::findFont(const char *fontFile) const
 }
 
 
-std::string TTFFinder::nxFindFont(const char *fontFile) const
+#ifdef __APPLE__
+std::string TTFFinder::macFindFont(const char *fontFile) 
 {
 	//This is a list of possible target dirs to search
 	//(Oh look Ma, I'm autoconf!)
 	const char *dirs[] = {	".",
-				"/usr/share/fonts/truetype",
-				"/usr/local/share/fonts/truetype",
+				"/Library/Fonts",
+				"" ,
+				}; //MUST end with "".
+
+	wxPathList *p = new wxPathList;
+
+	unsigned int ui=0;
+	//Try a few standard locations
+	while(strlen(dirs[ui]))
+	{
+		p->Add(wxCStr(dirs[ui]));
+		ui++;
+	};
+
+	wxString s;
+
+	//execute the search for the file
+	s= p->FindValidPath(wxCStr(fontFile));
+
+
+	std::string res;
+	if(s.size())
+	{
+		if(p->EnsureFileAccessible(s))
+			res = stlStr(s);
+	}
+
+	delete p;
+	return res;
+}
+#elif defined __UNIX_LIKE__ || defined __linux__
+std::string TTFFinder::nxFindFont(const char *fontFile) 
+{
+	//This is a list of possible target dirs to search
+	//(Oh look Ma, I'm autoconf!)
+
+	const char *dirs[] = {	".",
+				"/usr/share/fonts/truetype", //Old debian 
+				"/usr/share/fonts/truetype/freefont", // New debian
+				"/usr/share/fonts/truetype/ttf-dejavu", //New debian
+				"/usr/local/share/fonts/truetype", // User fonts
 				"/usr/X11R6/lib/X11/fonts/truetype",
 				"/usr/X11R6/lib64/X11/fonts/truetype",
-				"/usr/lib/X11/fonts/truetype",
-				"/usr/lib64/X11/fonts/truetype",
-				"/usr/local/lib/X11/fonts/truetype",
-				"/usr/local/lib64/X11/fonts/truetype",
+				"/usr/lib/X11/fonts/truetype",// Fedora 32
+				"/usr/lib64/X11/fonts/truetype", //Fedora 64
+				"/usr/local/lib/X11/fonts/truetype", // Fedora 32 new
+				"/usr/local/lib64/X11/fonts/truetype",// Fedora 64 new
 				"",
 				}; //MUST end with "".
 
@@ -677,8 +965,8 @@ std::string TTFFinder::nxFindFont(const char *fontFile) const
 	delete p;
 	return res;
 }
-
-std::string TTFFinder::winFindFont(const char *fontFile) const
+#elif defined  __WINDOWS__
+std::string TTFFinder::winFindFont(const char *fontFile)
 {
             //This is a list of possible target dirs to search
 	//(Oh look Ma, I'm autoconf!)
@@ -717,44 +1005,9 @@ std::string TTFFinder::winFindFont(const char *fontFile) const
 
 
 }
+#endif
 
-std::string TTFFinder::macFindFont(const char *fontFile) const
-{
-	//This is a list of possible target dirs to search
-	//(Oh look Ma, I'm autoconf!)
-	const char *dirs[] = {	".",
-				"/Library/Fonts",
-				"" ,
-				}; //MUST end with "".
-
-	wxPathList *p = new wxPathList;
-
-	unsigned int ui=0;
-	//Try a few standard locations
-	while(strlen(dirs[ui]))
-	{
-		p->Add(wxCStr(dirs[ui]));
-		ui++;
-	};
-
-	wxString s;
-
-	//execute the search for the file
-	s= p->FindValidPath(wxCStr(fontFile));
-
-
-	std::string res;
-	if(s.size())
-	{
-		if(p->EnsureFileAccessible(s))
-			res = stlStr(s);
-	}
-
-	delete p;
-	return res;
-}
-
-std::string TTFFinder::suggestFontName(unsigned int fontType, unsigned int index) const
+std::string TTFFinder::suggestFontName(unsigned int fontType, unsigned int index) 
 {
 	//Possible font names
 	const char *sansFontNames[] = {
