@@ -1,6 +1,6 @@
 /*
  * 	wxcomponents.h - custom wxwidgets components
- *	Copyright (C) 2011, D. Haley
+ *	Copyright (C) 2013, D. Haley
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -20,18 +20,33 @@
 #ifndef WXCOMPONENTS_H
 #define WXCOMPONENTS_H
 
-#include <wx/dc.h>
-#include <wx/settings.h>
 #include <wx/grid.h>
 #include <wx/treectrl.h>
 #include <wx/laywin.h>
-#include <wx/filedlg.h>
 
 #include <vector>
 #include <string>
 
-#include "assertion.h"
-#include "commonConstants.h"
+
+#include "backend/filtertree.h"
+
+//Shut wxwidgets assertion errors up by defining a "safe" cb_sort wrapper macro
+#if defined(__WXMAC__) || defined(__WXGTK20__)
+	       
+    #include <wx/version.h>
+    #if wxCHECK_VERSION(2,9,0)
+    //Sorted combos not supported under gtk in 2.8 series 
+    // http://trac.wxwidgets.org/ticket/4398
+    //and not supported in mac.
+    // http://trac.wxwidgets.org/ticket/12419
+        #define SAFE_CB_SORT 0 
+    #else
+        #define SAFE_CB_SORT wxCB_SORT
+	#endif
+#else
+	#define SAFE_CB_SORT wxCB_SORT
+#endif
+
 
 //!3D combo grid renderer, from
 //http://nomadsync.cvs.sourceforge.net/nomadsync/nomadsync/src/EzGrid.cpp?view=markup (GPL)
@@ -69,12 +84,16 @@ protected:
 	wxPoint m_pointActivate;
 };
 
+//!Update a wxTree control to layout according to the specified filter tree
+void upWxTreeCtrl(const FilterTree &filterTree, wxTreeCtrl *t,
+		std::map<size_t,Filter *> &filterMap,vector<const Filter *> &persistentFilters,
+		const Filter *visibleFilt);
 
 //!Data container for tree object data
 class wxTreeUint : public wxTreeItemData
 {
 	public:
-		wxTreeUint(unsigned int v) { value=v;};
+		wxTreeUint(unsigned int v) : value(v) {};
 		unsigned int value;
 };
 
@@ -93,6 +112,7 @@ struct GRID_PROPERTY
 	int renderPosition;
 	std::string name;
 	std::string data; //String version of data stored
+	std::string helpText; //Hover tool-tip help text
 };
 
 class wxPropertyGrid;
@@ -111,28 +131,38 @@ class wxPropertyGrid : public wxGrid
 		std::vector<std::vector<GRID_PROPERTY> > propertyKeys;
 		//Names of each of the grouped keys in propertyKeys
 		std::vector<std::string> sectionNames;
+
+		//Fit the columns to the specified size
 		void fitCols(wxSize &size);
+
+		//Rows used for separators
+		std::vector<int> sepRows;
+		//The last coordinates of mouse hovering 
+		size_t lastGridHoverCol,lastGridHoverRow;
+
+		//Is the nth row a separator row?
+		bool isSeparatorRow(int row) const;
 	public:
 		wxPropertyGrid(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, 
 					const wxSize& size = wxDefaultSize, long style = wxWANTS_CHARS);
+		~wxPropertyGrid();
+		
 		void OnSize(wxSizeEvent &size);
 		void OnLabelDClick(wxGridEvent &size);
-		~wxPropertyGrid();
+		void OnMouseMove(wxMouseEvent &mouse);
 		void clearKeys();
 
-		//!Set the number of, er, sets.
-		void setNumSets(unsigned int newSetCount){propertyKeys.resize(newSetCount); sectionNames.resize(newSetCount);};
+		//!Set the number of separator groups to use 
+		void setNumGroups(unsigned int newGroupCount){propertyKeys.resize(newGroupCount); sectionNames.resize(newGroupCount);};
 
-		//Set the names for each set. (sorry :()
-		void setSetName(unsigned int set, const std::string &name) {ASSERT(set < sectionNames.size()); sectionNames[set]=name;};
+		//Set the names for each group. This will appear as a small text in the blank rows
+		void setGroupName(unsigned int set, const std::string &name); 
 
-		//!Retreive the position of a cell, given key and set
-		void getPropertyCell(unsigned int set, unsigned int key,
-					unsigned int &xPos, unsigned int &yPos) const ;
-		//!This adds the item to the property key vector of a specified set
-		//!	key must be unique within a given set.
-		void addKey(const std::string &name, unsigned int set,
-				unsigned int newKey, unsigned int type, const std::string &data);
+		//!This adds the item to the property key vector of a specified group 
+		//!	key must be unique 
+		void addKey(const std::string &name, unsigned int group,
+				unsigned int newKey, unsigned int type, 
+				const std::string &data,const std::string &helpText);
 		
 		//!Click event
 		virtual void OnCellLeftClick(wxGridEvent &e);
@@ -140,17 +170,14 @@ class wxPropertyGrid : public wxGrid
 		//!Cause grid to layout its elements
 		void propertyLayout();
 
-		//!Get the set from the row
-		unsigned int getSetFromRow(int row) const;
-
 		//!Get the key from the row
 		unsigned int getKeyFromRow(int row) const;
 
-		//!Get the type form the row
+		//!Get the type from the row
 		unsigned int getTypeFromRow(int row) const;
 
 		//!Get the property from key and set 
-		const GRID_PROPERTY* getProperty(unsigned int set, unsigned int key) const;
+		const GRID_PROPERTY* getProperty(unsigned int key) const;
 
 		//!Clear the grid
 		void clear();
@@ -173,16 +200,17 @@ public:
 
 	void currentCell();
 	void selectData();
-	//!Copy data to the clipboard (not working? GTK???)
+	//!Copy data to the clipboard 
 	void copyData();
-	//!Prompts user to save data to file, and then saves it. pops up error dialog box  if there is a problem. Data is tab deliminated
+	//!Prompts user to save data to file, and then saves it. pops up error 
+	// dialog box if there is a problem. Data is tab deliminated
 	void saveData();
-//	void pasteData();
-//	void deleteData();
 
 	virtual void OnKey(wxKeyEvent &evt);
 
 	virtual ~CopyGrid(){};
+		
+	DECLARE_EVENT_TABLE()
 
 };
 
@@ -200,20 +228,20 @@ class TTFFinder
 {
 	private:
 		//*n?x (FHS compliant) searching
-		std::string nxFindFont(const char *fontFile) const;
+		static std::string nxFindFont(const char *fontFile);
 		//MS win. searching
-		std::string winFindFont(const char *fontFile) const;
+		static std::string winFindFont(const char *fontFile);
 		//Mac OS X searching
-		std::string macFindFont(const char *fontFile) const;
+		static std::string macFindFont(const char *fontFile);
 	public:
 		//Given a ttf file name, search for it in several common paths
-		std::string findFont(const char *fontFile) const;
+		static std::string findFont(const char *fontFile);
 
 		//!Given an font type (Sans, serif etc) suggest a font name. 
 		//As long as function does not return empty std::string,, then index+1 is a valid
 		//query (which may return empty std::string). Font names returned are a suggestion
 		//only. Pass to findFont to confirm that a font file exists.
-		std::string suggestFontName(unsigned int fontType, unsigned int index) const ;
+		static std::string suggestFontName(unsigned int fontType, unsigned int index) ;
 
 		//!Returns a valid file that points to an installed ttf file, or an empty string
 		//NOTE: TTF file is not checked for content validity!
